package org.eclipse.equinox.launcher;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.dna.launcher.Constants;
import com.jiuqi.dna.launcher.ServerDaemon;
import com.jiuqi.util.ArrayUtils;
import com.jiuqi.util.BundleUtils;
import com.jiuqi.util.StringUtil;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.security.CodeSource;
import java.security.Policy;
import java.security.ProtectionDomain;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import org.eclipse.equinox.launcher.IORedirecter;
import org.eclipse.equinox.launcher.JNIBridge;

public class Main implements Constants {
  protected boolean debug = false;
  
  protected String bootLocation = null;
  
  protected URL installLocation = null;
  
  protected URL configurationLocation = null;
  
  protected String parentConfigurationLocation = null;
  
  protected String framework = "org.eclipse.osgi";
  
  protected String devClassPath = null;
  
  private Properties devClassPathProps = null;
  
  protected boolean inDevelopmentMode = false;
  
  protected String os = null;
  
  protected String ws = null;
  
  protected String arch = null;
  
  private String library = null;
  
  private String exitData = null;
  
  private String vm = null;
  
  private String[] vmargs = null;
  
  private String[] commands = null;
  
  String[] extensionPaths = null;
  
  JNIBridge bridge = null;
  
  private boolean showSplash = false;
  
  private String splashLocation = null;
  
  private String endSplash = null;
  
  private boolean initialize = false;
  
  private boolean splashDown = false;
  
  private final Thread splashHandler = (Thread)new SplashHandler(this);
  
  public static final String SPLASH_HANDLE = "org.eclipse.equinox.launcher.splash.handle";
  
  public static final String SPLASH_LOCATION = "org.eclipse.equinox.launcher.splash.location";
  
  private static final String FRAMEWORK = "-framework";
  
  private static final String INSTALL = "-install";
  
  private static final String INITIALIZE = "-initialize";
  
  private static final String VM = "-vm";
  
  private static final String VMARGS = "-vmargs";
  
  private static final String DEBUG = "-debug";
  
  private static final String DEV = "-dev";
  
  private static final String CONFIGURATION = "-configuration";
  
  private static final String NOSPLASH = "-nosplash";
  
  private static final String SHOWSPLASH = "-showsplash";
  
  private static final String EXITDATA = "-exitdata";
  
  private static final String NAME = "-name";
  
  private static final String LAUNCHER = "-launcher";
  
  private static final String LIBRARY = "--launcher.library";
  
  private static final String NL = "-nl";
  
  private static final String ENDSPLASH = "-endsplash";
  
  private static final String SPLASH_IMAGE = "splash.bmp";
  
  private static final String CLEAN = "-clean";
  
  private static final String NOEXIT = "-noExit";
  
  private static final String OS = "-os";
  
  private static final String WS = "-ws";
  
  private static final String REDIRECT = "-redirect";
  
  private static final String ARCH = "-arch";
  
  private static final String STARTUP = "-startup";
  
  private static final String OSGI = "org.eclipse.osgi";
  
  private static final String STARTER = "org.eclipse.core.runtime.adaptor.EclipseStarter";
  
  private static final String PLATFORM_URL = "platform:/base/";
  
  private static final String ECLIPSE_PROPERTIES = "eclipse.properties";
  
  private static final String FILE_SCHEME = "file:";
  
  protected static final String REFERENCE_SCHEME = "reference:";
  
  protected static final String JAR_SCHEME = "jar:";
  
  private static final String CONFIG_DIR = "configuration/";
  
  private static final String CONFIG_FILE = "config.ini";
  
  private static final String CONFIG_FILE_TEMP_SUFFIX = ".tmp";
  
  private static final String CONFIG_FILE_BAK_SUFFIX = ".bak";
  
  private static final String ECLIPSE = "eclipse";
  
  private static final String PRODUCT_SITE_MARKER = ".eclipseproduct";
  
  private static final String PRODUCT_SITE_ID = "id";
  
  private static final String PRODUCT_SITE_VERSION = "version";
  
  private static final String PROP_USER_HOME = "user.home";
  
  private static final String PROP_USER_DIR = "user.dir";
  
  private static final String PROP_INSTALL_AREA = "osgi.install.area";
  
  private static final String PROP_OS_TYPE = "os.type";
  
  private static final String PROP_REDIRECT_OUT = "out.redirect";
  
  private static final String PROP_CONSOLE_BLOCK = "osgi.console.blockOnReady";
  
  private static final String PROP_CONFIG_AREA = "osgi.configuration.area";
  
  private static final String PROP_CONFIG_AREA_DEFAULT = "osgi.configuration.area.default";
  
  private static final String PROP_BASE_CONFIG_AREA = "osgi.baseConfiguration.area";
  
  private static final String PROP_SHARED_CONFIG_AREA = "osgi.sharedConfiguration.area";
  
  private static final String PROP_CONFIG_CASCADED = "osgi.configuration.cascaded";
  
  protected static final String PROP_FRAMEWORK = "osgi.framework";
  
  private static final String PROP_SPLASHPATH = "osgi.splashPath";
  
  private static final String PROP_SPLASHLOCATION = "osgi.splashLocation";
  
  private static final String PROP_CLASSPATH = "osgi.frameworkClassPath";
  
  private static final String PROP_EXTENSIONS = "osgi.framework.extensions";
  
  private static final String PROP_FRAMEWORK_SYSPATH = "osgi.syspath";
  
  private static final String PROP_FRAMEWORK_SHAPE = "osgi.framework.shape";
  
  private static final String PROP_LOGFILE = "osgi.logfile";
  
  private static final String PROP_REQUIRED_JAVA_VERSION = "osgi.requiredJavaVersion";
  
  private static final String PROP_PARENT_CLASSLOADER = "osgi.parentClassloader";
  
  private static final String PROP_FRAMEWORK_PARENT_CLASSLOADER = "osgi.frameworkParentClassloader";
  
  private static final String PROP_NL = "osgi.nl";
  
  static final String PROP_NOSHUTDOWN = "osgi.noShutdown";
  
  private static final String PROP_DEBUG = "osgi.debug";
  
  private static final String PROP_EXITCODE = "eclipse.exitcode";
  
  private static final String PROP_EXITDATA = "eclipse.exitdata";
  
  private static final String PROP_VM = "eclipse.vm";
  
  private static final String PROP_VMARGS = "eclipse.vmargs";
  
  private static final String PROP_COMMANDS = "eclipse.commands";
  
  private static final String PROP_ECLIPSESECURITY = "eclipse.security";
  
  private static final String NONE = "@none";
  
  private static final String NO_DEFAULT = "@noDefault";
  
  private static final String USER_HOME = "@user.home";
  
  private static final String USER_DIR = "@user.dir";
  
  private static final String PARENT_CLASSLOADER_APP = "app";
  
  private static final String PARENT_CLASSLOADER_EXT = "ext";
  
  private static final String PARENT_CLASSLOADER_BOOT = "boot";
  
  private static final String PARENT_CLASSLOADER_CURRENT = "current";
  
  protected static final String SESSION = "!SESSION";
  
  protected static final String ENTRY = "!ENTRY";
  
  protected static final String MESSAGE = "!MESSAGE";
  
  protected static final String STACK = "!STACK";
  
  protected static final int ERROR = 4;
  
  protected static final String PLUGIN_ID = "com.jiuqi.dna.launcher";
  
  protected File logFile = null;
  
  protected BufferedWriter log = null;
  
  protected boolean newSession = true;
  
  private String getWS() {
    if (this.ws != null)
      return this.ws; 
    String os = getOS();
    if (os.equals("win32"))
      return "win32"; 
    if (os.equals("linux"))
      return "gtk"; 
    if (os.equals("macosx"))
      return "carbon"; 
    if (os.equals("hpux"))
      return "motif"; 
    if (os.equals("aix"))
      return "motif"; 
    if (os.equals("solaris"))
      return "motif"; 
    if (os.equals("qnx"))
      return "photon"; 
    return "unknown";
  }
  
  private String getOS() {
    if (this.os != null)
      return this.os; 
    String osName = System.getProperties().getProperty("os.name");
    if (osName.regionMatches(true, 0, "win32", 0, 3))
      return "win32"; 
    if (osName.equalsIgnoreCase("SunOS"))
      return "solaris"; 
    if (osName.equalsIgnoreCase("Linux"))
      return "linux"; 
    if (osName.equalsIgnoreCase("QNX"))
      return "qnx"; 
    if (osName.equalsIgnoreCase("AIX"))
      return "aix"; 
    if (osName.equalsIgnoreCase("HP-UX"))
      return "hpux"; 
    if (osName.regionMatches(true, 0, "Mac OS", 0, 
        "Mac OS".length()))
      return "macosx"; 
    return "unknown";
  }
  
  private String getArch() {
    if (this.arch != null)
      return this.arch; 
    String name = System.getProperties().getProperty("os.arch");
    if (name.equalsIgnoreCase("i386"))
      return "x86"; 
    if (name.equalsIgnoreCase("amd64"))
      return "x86_64"; 
    return name;
  }
  
  private void setupJNI(URL[] defaultPath) {
    String libPath = null;
    if (this.library != null) {
      File lib = new File(this.library);
      if (lib.isDirectory()) {
        libPath = searchFor("eclipse", lib.getAbsolutePath());
      } else if (lib.exists()) {
        libPath = lib.getAbsolutePath();
      } 
    } 
    if (libPath == null) {
      String fragmentOS = getOS();
      StringBuffer buffer = new StringBuffer("com.jiuqi.dna.launcher");
      buffer.append('.');
      buffer.append(getWS());
      buffer.append('.');
      buffer.append(fragmentOS);
      if (!fragmentOS.equals("macosx")) {
        buffer.append('.');
        buffer.append(getArch());
      } 
      String fragmentName = buffer.toString();
      String fragment = null;
      if (this.inDevelopmentMode) {
        String devPathList = this.devClassPathProps
          .getProperty("com.jiuqi.dna.launcher");
        String[] locations = getArrayFromList(devPathList);
        if (locations.length > 0) {
          File location = new File(locations[0]);
          if (location.isAbsolute()) {
            String dir = location.getParent();
            fragment = searchFor(fragmentName, dir);
            if (fragment != null)
              libPath = searchFor("eclipse", fragment); 
          } 
        } 
      } 
      if (libPath == null && this.bootLocation != null) {
        URL[] urls = defaultPath;
        if (urls != null && urls.length > 0)
          for (int i = urls.length - 1; i >= 0 && libPath == null; i--) {
            File entryFile = new File(urls[i].getFile());
            String dir = entryFile.getParent();
            if (this.inDevelopmentMode) {
              String devDir = String.valueOf(dir) + 
                "/" + "com.jiuqi.dna.launcher" + "/fragments";
              fragment = searchFor(fragmentName, devDir);
            } 
            if (fragment == null)
              fragment = searchFor(fragmentName, dir); 
            if (fragment != null)
              libPath = searchFor("eclipse", fragment); 
          }  
      } 
      if (libPath == null) {
        URL install = getInstallLocation();
        String location = install.getFile();
        location = String.valueOf(location) + "/bundles/";
        if ((new File(location)).exists()) {
          fragment = searchFor(fragmentName, location);
        } else {
          location = String.valueOf(location) + "/thr/";
          fragment = searchFor(fragmentName, location);
        } 
        if (fragment != null)
          libPath = searchFor("eclipse", fragment); 
      } 
    } 
    this.library = libPath;
    if (this.library != null)
      this.bridge = new JNIBridge(this.library); 
  }
  
  protected void basicRun(String[] args) throws Exception {
    System.getProperties().put(
        "eclipse.startTime", Long.toString(System.currentTimeMillis()));
    String[] args2 = { "-os", System.getProperty("os.name"), "-console" };
    this.commands = args2;
    String[] passThruArgs = processCommandLine(args2);
    if (!this.debug)
      this.debug = (System.getProperty("osgi.debug") != null); 
    setupVMProperties();
    getInstallLocation();
    String installPath = System.getProperty("osgi.install.area");
    if (installPath != null) {
      URL url = new URL(installPath);
      File f = new File(url.getFile());
      installPath = f.getAbsolutePath();
      System.setProperty("com.jiuqi.dna.rootpath", installPath);
      System.getProperties().put("osgi.configuration.area", String.valueOf(installPath) + "/work");
    } 
    processConfiguration2();
    if (args.length > 0 && "stop".equals(args[0])) {
      ServerDaemon.stopServer();
    } else if (args.length > 0 && "restart".equals(args[0])) {
      ServerDaemon.restartServer();
    } else {
      ServerDaemon.startServer();
    } 
    if (Boolean.getBoolean("out.redirect"))
      IORedirecter.redirectIO(installPath); 
    if (Boolean.getBoolean("osgi.console.blockOnReady"))
      System.setProperty("osgi.console.blockOnReady", ""); 
    URL[] bootPath = getBootPath(this.bootLocation);
    setupJNI(bootPath);
    if (!checkVersion(System.getProperty("java.version"), 
        System.getProperty("osgi.requiredJavaVersion")))
      return; 
    setSecurityPolicy(bootPath);
    handleSplash(bootPath);
    beforeFwkInvocation();
    invokeFramework(passThruArgs, bootPath);
  }
  
  protected void beforeFwkInvocation() {}
  
  private final void processConfiguration2() {
    System.setProperty("osgi.clean", "true");
    System.setProperty("org.eclipse.equinox.http.jetty.autostart", "false");
    System.setProperty("POOL_MAX", "100000");
    System.setProperty("osgi.noShutdown", "true");
    StringBuffer initBundles = new StringBuffer();
    String serverRoot = System.getProperty("com.jiuqi.dna.rootpath");
    File f = new File(serverRoot);
    String serverRootPath = StringUtil.formatPath(f.getPath(), false);
    if (serverRootPath.endsWith("/"))
      serverRootPath = serverRootPath.substring(0, serverRootPath.length() - 1); 
    byte b;
    int i;
    String[] arrayOfString;
    for (i = (arrayOfString = bundleDirs).length, b = 0; b < i; ) {
      String bundleDir = arrayOfString[b];
      File fBundleDir = new File(serverRoot, bundleDir);
      doStruct(fBundleDir, initBundles, serverRootPath);
      b++;
    } 
    if (Boolean.getBoolean("launcher.debug"))
      System.out.println(initBundles.toString()); 
    System.setProperty("osgi.bundles", initBundles.toString());
  }
  
  private static final String[] bundleDirs = new String[] { "app", "dna", "thr", "lib", "bundles" };
  
  private static final String prefix2 = "reference:file:..";
  
  private static final String SUBFIX_START = "@start";
  
  private static final String[] notLoadBundles = new String[] { "org.eclipse.osgi", 
      "com.jiuqi.dna.launcher" };
  
  private final void doStruct(File fBundleDir, StringBuffer configBuffer, String serverRootPath) {
    if (BundleUtils.isBundle(fBundleDir)) {
      String bundleId = BundleUtils.getBundleId(fBundleDir);
      if (isNotLoadBundles(bundleId))
        return; 
      String filePath = StringUtil.formatPath(fBundleDir.getPath(), false);
      filePath = filePath.substring(serverRootPath.length());
      if (canStartBundle(fBundleDir)) {
        filePath = "reference:file:.." + filePath + "@start";
      } else {
        filePath = "reference:file:.." + filePath;
      } 
      configBuffer.append(filePath);
      configBuffer.append(",");
    } else if (fBundleDir.isDirectory()) {
      byte b;
      int i;
      File[] arrayOfFile;
      for (i = (arrayOfFile = fBundleDir.listFiles()).length, b = 0; b < i; ) {
        File f = arrayOfFile[b];
        doStruct(f, configBuffer, serverRootPath);
        b++;
      } 
    } 
  }
  
  private final boolean isNotLoadBundles(String bundleId) {
    return ArrayUtils.contains((Object[])notLoadBundles, bundleId);
  }
  
  private boolean canStartBundle(File bundleFile) {
    return !BundleUtils.isFragmentBundle(bundleFile);
  }
  
  protected void setSecurityPolicy(URL[] bootPath) {
    String eclipseSecurity = System.getProperty("eclipse.security");
    if (eclipseSecurity != null) {
      SecurityManager sm = System.getSecurityManager();
      boolean setSM = false;
      if (sm == null) {
        if (eclipseSecurity.length() < 1)
          eclipseSecurity = "java.lang.SecurityManager"; 
        try {
          Class<?> clazz = Class.forName(eclipseSecurity);
          sm = (SecurityManager)clazz.newInstance();
          setSM = true;
        } catch (Throwable throwable) {
          System.getProperties().put("java.security.manager", 
              eclipseSecurity);
        } 
      } 
      ProtectionDomain domain = org.eclipse.equinox.launcher.Main.class.getProtectionDomain();
      CodeSource source = null;
      if (domain != null)
        source = org.eclipse.equinox.launcher.Main.class.getProtectionDomain().getCodeSource(); 
      if (domain == null || source == null) {
        log("Can not automatically set the security manager. Please use a policy file.");
        return;
      } 
      URL[] rootURLs = new URL[bootPath.length + 1];
      rootURLs[0] = source.getLocation();
      System.arraycopy(bootPath, 0, rootURLs, 1, bootPath.length);
      EclipsePolicy eclipsePolicy = new EclipsePolicy(this, Policy.getPolicy(), 
          rootURLs);
      Policy.setPolicy((Policy)eclipsePolicy);
      if (setSM)
        System.setSecurityManager(sm); 
    } 
  }
  
  private void invokeFramework(String[] passThruArgs, URL[] bootPath) throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException, Error, Exception, InvocationTargetException {
    String type = System.getProperty("osgi.frameworkParentClassloader", 
        System.getProperty("osgi.parentClassloader", 
          "boot"));
    ClassLoader parent = null;
    if ("app".equalsIgnoreCase(type)) {
      parent = ClassLoader.getSystemClassLoader();
    } else if ("ext".equalsIgnoreCase(type)) {
      ClassLoader appCL = ClassLoader.getSystemClassLoader();
      if (appCL != null)
        parent = appCL.getParent(); 
    } else if ("current".equalsIgnoreCase(type)) {
      parent = getClass().getClassLoader();
    } 
    StartupClassLoader startupClassLoader = new StartupClassLoader(this, bootPath, parent);
    Class<?> clazz = startupClassLoader.loadClass("org.eclipse.core.runtime.adaptor.EclipseStarter");
    Method method = clazz.getDeclaredMethod(
        "run", new Class[] { String[].class, Runnable.class });
    try {
      method.invoke(clazz, new Object[] { passThruArgs, 
            this.splashHandler });
    } catch (InvocationTargetException e) {
      if (e.getTargetException() instanceof Error)
        throw (Error)e.getTargetException(); 
      if (e.getTargetException() instanceof Exception)
        throw (Exception)e.getTargetException(); 
      throw e;
    } 
  }
  
  private boolean checkVersion(String availableVersion, String requiredVersion) {
    if (requiredVersion == null || availableVersion == null)
      return true; 
    try {
      Identifier required = new Identifier(requiredVersion);
      Identifier available = new Identifier(availableVersion);
      boolean compatible = available.isGreaterEqualTo(required);
      if (!compatible) {
        System.getProperties().put("eclipse.exitcode", "14");
        System.getProperties()
          .put(
            "eclipse.exitdata", 
            "<title>Incompatible JVM</title>Version " + availableVersion + " of the JVM is not suitable for this product. Version: " + requiredVersion + " or greater is required.");
      } 
      return compatible;
    } catch (SecurityException securityException) {
      return true;
    } catch (NumberFormatException numberFormatException) {
      return true;
    } 
  }
  
  protected String decode(String urlString) {
    try {
      Class<URLDecoder> clazz = URLDecoder.class;
      Method method = clazz.getDeclaredMethod(
          "decode", new Class[] { String.class, String.class });
      if (urlString.indexOf('+') >= 0) {
        int len = urlString.length();
        StringBuffer buf = new StringBuffer(len);
        for (int j = 0; j < len; j++) {
          char c = urlString.charAt(j);
          if (c == '+') {
            buf.append("%2B");
          } else {
            buf.append(c);
          } 
        } 
        urlString = buf.toString();
      } 
      Object result = method.invoke(null, new Object[] { urlString, 
            "UTF-8" });
      if (result != null)
        return (String)result; 
    } catch (Exception exception) {}
    boolean replaced = false;
    byte[] encodedBytes = urlString.getBytes();
    int encodedLength = encodedBytes.length;
    byte[] decodedBytes = new byte[encodedLength];
    int decodedLength = 0;
    for (int i = 0; i < encodedLength; i++) {
      byte b = encodedBytes[i];
      if (b == 37) {
        if (i + 2 >= encodedLength)
          throw new IllegalArgumentException(
              "Malformed URL (\"" + urlString + "\"): % must be followed by 2 digits."); 
        byte enc1 = encodedBytes[++i];
        byte enc2 = encodedBytes[++i];
        b = (byte)((hexToByte(enc1) << 4) + hexToByte(enc2));
        replaced = true;
      } 
      decodedBytes[decodedLength++] = b;
    } 
    if (!replaced)
      return urlString; 
    try {
      return new String(decodedBytes, 0, decodedLength, "UTF-8");
    } catch (UnsupportedEncodingException unsupportedEncodingException) {
      return new String(decodedBytes, 0, decodedLength);
    } 
  }
  
  protected String[] getArrayFromList(String prop) {
    if (prop == null || prop.trim().equals(""))
      return new String[0]; 
    Vector<String> list = new Vector();
    StringTokenizer tokens = new StringTokenizer(prop, ",");
    while (tokens.hasMoreTokens()) {
      String token = tokens.nextToken().trim();
      if (!token.equals(""))
        list.addElement(token); 
    } 
    return list.isEmpty() ? new String[0] : list
      .<String>toArray(new String[list.size()]);
  }
  
  private URL[] getDevPath(URL base) throws IOException {
    ArrayList result = new ArrayList(5);
    if (this.inDevelopmentMode)
      addDevEntries(base, result, "org.eclipse.osgi"); 
    addBaseJars(base, result);
    return (URL[])result.toArray((Object[])new URL[result.size()]);
  }
  
  URL constructURL(URL url, String name) {
    String externalForm = url.toExternalForm();
    if (externalForm.endsWith(".jar"))
      try {
        return new URL("jar:" + url + "!/" + name);
      } catch (MalformedURLException malformedURLException) {} 
    try {
      return new URL(url, name);
    } catch (MalformedURLException malformedURLException) {
      return null;
    } 
  }
  
  private void readFrameworkExtensions(URL base, ArrayList result) throws IOException {
    String[] extensions = getArrayFromList(System.getProperties()
        .getProperty("osgi.framework.extensions"));
    String parent = (new File(base.getFile())).getParent().toString();
    ArrayList<String> extensionResults = new ArrayList(extensions.length);
    for (int i = 0; i < extensions.length; i++) {
      String path = searchFor(extensions[i], parent);
      if (path == null) {
        log("Could not find extension: " + extensions[i]);
      } else {
        String qualifiedPath;
        if (this.debug)
          System.out.println("Loading extension: " + extensions[i]); 
        URL extensionURL = null;
        if (this.installLocation.getProtocol().equals("file")) {
          extensionResults.add(path);
          extensionURL = (new File(path)).toURL();
        } else {
          extensionURL = new URL(this.installLocation.getProtocol(), 
              this.installLocation.getHost(), this.installLocation
              .getPort(), path);
        } 
        Properties extensionProperties = null;
        try {
          extensionProperties = loadProperties(constructURL(
                extensionURL, "eclipse.properties"));
        } catch (IOException iOException) {
          if (this.debug)
            System.out
              .println("\teclipse.properties not found"); 
        } 
        String extensionClassPath = null;
        if (extensionProperties != null) {
          extensionClassPath = extensionProperties
            .getProperty("osgi.frameworkClassPath");
        } else {
          extensionProperties = new Properties();
        } 
        (new String[1])[0] = "";
        String[] entries = (extensionClassPath == null || extensionClassPath.length() == 0) ? new String[1] : getArrayFromList(extensionClassPath);
        if (System.getProperty("osgi.frameworkClassPath") == null) {
          qualifiedPath = ".";
        } else {
          qualifiedPath = "";
        } 
        for (int j = 0; j < entries.length; j++)
          qualifiedPath = String.valueOf(qualifiedPath) + ", file:" + path + entries[j]; 
        extensionProperties.put("osgi.frameworkClassPath", qualifiedPath);
        mergeProperties(System.getProperties(), extensionProperties);
        if (this.inDevelopmentMode)
          addDevEntries(extensionURL, result, extensions[i]); 
      } 
    } 
    this.extensionPaths = extensionResults
      .<String>toArray(new String[extensionResults.size()]);
  }
  
  private void addBaseJars(URL base, ArrayList result) throws IOException {
    String baseJarList = System.getProperty("osgi.frameworkClassPath");
    if (baseJarList == null) {
      readFrameworkExtensions(base, result);
      baseJarList = System.getProperties().getProperty("osgi.frameworkClassPath");
    } 
    File fwkFile = new File(base.getFile());
    boolean fwkIsDirectory = fwkFile.isDirectory();
    if (fwkIsDirectory) {
      System.getProperties().put("osgi.framework.shape", "folder");
    } else {
      System.getProperties().put("osgi.framework.shape", "jar");
    } 
    String fwkPath = (new File((new File(base.getFile())).getParent()))
      .getAbsolutePath();
    if (Character.isUpperCase(fwkPath.charAt(0))) {
      char[] chars = fwkPath.toCharArray();
      chars[0] = Character.toLowerCase(chars[0]);
      fwkPath = new String(chars);
    } 
    System.getProperties().put("osgi.syspath", fwkPath);
    String[] baseJars = getArrayFromList(baseJarList);
    if (baseJars.length == 0) {
      if (!this.inDevelopmentMode && (
        new File(base.getFile())).isDirectory())
        throw new IOException("Unable to initialize osgi.frameworkClassPath"); 
      addEntry(base, result);
      return;
    } 
    for (int i = 0; i < baseJars.length; i++) {
      String string = baseJars[i];
      try {
        if (string.equals("."))
          addEntry(base, result); 
        URL url = null;
        if (string.startsWith("file:")) {
          url = (new File(string.substring(5))).toURL();
        } else {
          url = new URL(string);
        } 
        addEntry(url, result);
      } catch (MalformedURLException malformedURLException) {
        addEntry(new URL(base, string), result);
      } 
    } 
  }
  
  protected void addEntry(URL url, List<URL> result) {
    if ((new File(url.getFile())).exists())
      result.add(url); 
  }
  
  private void addDevEntries(URL base, List result, String symbolicName) throws MalformedURLException {
    if (this.devClassPathProps == null)
      return; 
    String devPathList = this.devClassPathProps.getProperty(symbolicName);
    if (devPathList == null)
      devPathList = this.devClassPathProps.getProperty("*"); 
    String[] locations = getArrayFromList(devPathList);
    for (int i = 0; i < locations.length; i++) {
      URL url;
      String location = locations[i];
      File path = new File(location);
      if (path.isAbsolute()) {
        url = path.toURL();
      } else {
        char lastChar = location.charAt(location.length() - 1);
        if (location.endsWith(".jar") || lastChar == '/' || lastChar == '\\') {
          url = new URL(base, location);
        } else {
          url = new URL(base, String.valueOf(location) + "/");
        } 
      } 
      addEntry(url, result);
    } 
  }
  
  protected URL[] getBootPath(String base) throws IOException {
    URL url = null;
    if (base != null) {
      url = buildURL(base, true);
    } else {
      url = getInstallLocation();
      String path = null;
      File dir = new File(url.getFile(), "bundles");
      if (dir.exists()) {
        path = dir.getAbsolutePath();
      } else {
        dir = new File(url.getFile(), "thr");
        if (dir.exists())
          path = dir.getAbsolutePath(); 
      } 
      path = searchFor(this.framework, path);
      if (path == null)
        throw new RuntimeException("Could not find framework"); 
      if (url.getProtocol().equals("file")) {
        url = (new File(path)).toURL();
      } else {
        url = new URL(url.getProtocol(), url.getHost(), url.getPort(), 
            path);
      } 
    } 
    if (System.getProperty("osgi.framework") == null)
      System.getProperties().put("osgi.framework", url.toExternalForm()); 
    if (this.debug)
      System.out
        .println("Framework located:\n    " + url.toExternalForm()); 
    URL[] result = getDevPath(url);
    if (this.debug) {
      System.out.println("Framework classpath:");
      for (int i = 0; i < result.length; i++)
        System.out.println("    " + result[i].toExternalForm()); 
    } 
    return result;
  }
  
  protected String searchFor(String target, String start) {
    return searchFor(target, null, start);
  }
  
  protected String searchFor(String target, String targetSuffix, String start) {
    String[] candidates = (new File(start)).list();
    if (candidates == null)
      return null; 
    ArrayList<String> matches = new ArrayList(2);
    for (int i = 0; i < candidates.length; i++) {
      if (candidates[i].equals(target) || 
        candidates[i].startsWith(String.valueOf(target) + "_"))
        matches.add(candidates[i]); 
    } 
    String[] names = matches.<String>toArray(new String[matches.size()]);
    int result = findMax(names);
    if (result == -1)
      return null; 
    File candidate = new File(start, names[result]);
    return String.valueOf(candidate.getAbsolutePath().replace(File.separatorChar, '/')) + (
      candidate.isDirectory() ? "/" : "");
  }
  
  protected int findMax(String[] candidates) {
    int result = -1;
    Object maxVersion = null;
    for (int i = 0; i < candidates.length; i++) {
      String name = candidates[i];
      String version = "";
      int index = name.indexOf('_');
      if (index != -1)
        version = name.substring(index + 1); 
      Object currentVersion = getVersionElements(version);
      if (maxVersion == null) {
        result = i;
        maxVersion = currentVersion;
      } else if (compareVersion((Object[])maxVersion, 
          (Object[])currentVersion) < 0) {
        result = i;
        maxVersion = currentVersion;
      } 
    } 
    return result;
  }
  
  private int compareVersion(Object[] left, Object[] right) {
    int result = ((Integer)left[0]).compareTo((Integer)right[0]);
    if (result != 0)
      return result; 
    result = ((Integer)left[1]).compareTo((Integer)right[1]);
    if (result != 0)
      return result; 
    result = ((Integer)left[2]).compareTo((Integer)right[2]);
    if (result != 0)
      return result; 
    return ((String)left[3]).compareTo((String)right[3]);
  }
  
  private Object[] getVersionElements(String version) {
    if (version.endsWith(".jar"))
      version = version.substring(0, version.length() - 4); 
    Object[] result = { new Integer(0), new Integer(0), new Integer(0), "" };
    StringTokenizer t = new StringTokenizer(version, ".");
    int i = 0;
    while (t.hasMoreTokens() && i < 4) {
      String token = t.nextToken();
      if (i < 3)
        try {
          result[i++] = new Integer(token);
          continue;
        } catch (Exception exception) {
          break;
        }  
      result[i++] = token;
    } 
    return result;
  }
  
  private static URL buildURL(String spec, boolean trailingSlash) {
    if (spec == null)
      return null; 
    boolean isFile = spec.startsWith("file:");
    try {
      if (isFile) {
        File toAdjust = new File(spec.substring(5));
        if (toAdjust.isDirectory())
          return adjustTrailingSlash(toAdjust.toURL(), trailingSlash); 
        return toAdjust.toURL();
      } 
      return new URL(spec);
    } catch (MalformedURLException malformedURLException) {
      if (isFile)
        return null; 
      try {
        File toAdjust = new File(spec);
        if (toAdjust.isDirectory())
          return adjustTrailingSlash(toAdjust.toURL(), trailingSlash); 
        return toAdjust.toURL();
      } catch (MalformedURLException malformedURLException1) {
        return null;
      } 
    } 
  }
  
  private static URL adjustTrailingSlash(URL url, boolean trailingSlash) throws MalformedURLException {
    String file = url.getFile();
    if (trailingSlash == file.endsWith("/"))
      return url; 
    file = trailingSlash ? (String.valueOf(file) + "/") : file.substring(0, file.length() - 1);
    return new URL(url.getProtocol(), url.getHost(), file);
  }
  
  private URL buildLocation(String property, URL defaultLocation, String userDefaultAppendage) {
    URL result = null;
    String location = System.getProperty(property);
    System.getProperties().remove(property);
    try {
    
    } finally {
      if (result != null)
        System.getProperties().put(property, result.toExternalForm()); 
    } 
    if (result != null)
      System.getProperties().put(property, result.toExternalForm()); 
    return result;
  }
  
  private String substituteVar(String source, String var, String prop) {
    String value = System.getProperty(prop, "");
    return String.valueOf(value) + source.substring(var.length());
  }
  
  private String computeDefaultConfigurationLocation() {
    URL install = getInstallLocation();
    if (install.getProtocol().equals("file")) {
      File installDir = new File(install.getFile());
      if (canWrite(installDir))
        return String.valueOf(installDir.getAbsolutePath()) + File.separator + 
          "configuration/"; 
    } 
    return computeDefaultUserAreaLocation("configuration/");
  }
  
  private static boolean canWrite(File installDir) {
    if (!installDir.canWrite())
      return false; 
    if (!installDir.isDirectory())
      return false; 
    File fileTest = null;
    try {
      fileTest = File.createTempFile("writtableArea", ".dll", installDir);
    } catch (IOException iOException) {
      return false;
    } finally {
      if (fileTest != null)
        fileTest.delete(); 
    } 
    return true;
  }
  
  private String computeDefaultUserAreaLocation(String pathAppendage) {
    int hashCode;
    URL installURL = getInstallLocation();
    if (installURL == null)
      return null; 
    File installDir = new File(installURL.getFile());
    try {
      hashCode = installDir.getCanonicalPath().hashCode();
    } catch (IOException iOException) {
      hashCode = installDir.getAbsolutePath().hashCode();
    } 
    if (hashCode < 0)
      hashCode = -hashCode; 
    String installDirHash = String.valueOf(hashCode);
    String appName = ".eclipse";
    File eclipseProduct = new File(installDir, ".eclipseproduct");
    if (eclipseProduct.exists()) {
      Properties props = new Properties();
      try {
        props.load(new FileInputStream(eclipseProduct));
        String appId = props.getProperty("id");
        if (appId == null || appId.trim().length() == 0)
          appId = "eclipse"; 
        String appVersion = props.getProperty("version");
        if (appVersion == null || appVersion.trim().length() == 0)
          appVersion = ""; 
        appName = String.valueOf(appName) + File.separator + appId + 
          "_" + appVersion + "_" + installDirHash;
      } catch (IOException iOException) {
        appName = String.valueOf(appName) + File.separator + installDirHash;
      } 
    } else {
      appName = String.valueOf(appName) + File.separator + installDirHash;
    } 
    String userHome = System.getProperty("user.home");
    return (new File(userHome, String.valueOf(appName) + "/" + pathAppendage)).getAbsolutePath();
  }
  
  public static void main(String argString) {
    Vector<Object> list = new Vector(5);
    StringTokenizer tokens = new StringTokenizer(argString, " ");
    while (tokens
      .hasMoreElements())
      list.addElement(tokens.nextElement()); 
    main(list.<String>toArray(new String[list.size()]));
  }
  
  public static void main(String[] args) {
    int result = 0;
    try {
      result = (new org.eclipse.equinox.launcher.Main()).run(args);
    } catch (Throwable t) {
      t.printStackTrace();
    } finally {}
  }
  
  public int run(String[] args) {
    int result = 0;
    try {
      basicRun(args);
      String exitCode = System.getProperty("eclipse.exitcode");
      try {
        result = (exitCode == null) ? 0 : Integer.parseInt(exitCode);
      } catch (NumberFormatException numberFormatException) {
        result = 17;
      } 
    } catch (Throwable e) {
      if (!"13".equals(System.getProperty("eclipse.exitcode"))) {
        log("Exception launching the Eclipse Platform:");
        log(e);
        String message = "An error has occurred";
        if (this.logFile == null) {
          message = String.valueOf(message) + " and could not be logged: \n" + e.getMessage();
        } else {
          message = String.valueOf(message) + ".  See the log file\n" + this.logFile.getAbsolutePath();
        } 
        System.getProperties().put("eclipse.exitdata", message);
      } 
      result = 13;
    } finally {
      takeDownSplash();
      if (this.bridge != null)
        this.bridge.uninitialize(); 
    } 
    System.getProperties().put("eclipse.exitcode", Integer.toString(result));
    setExitData();
    return result;
  }
  
  private void setExitData() {
    String data = System.getProperty("eclipse.exitdata");
    if (data == null || this.bridge == null)
      return; 
    this.bridge.setExitData(this.exitData, data);
  }
  
  protected String[] processCommandLine(String[] args) {
    if (args.length == 0)
      return args; 
    int[] configArgs = new int[args.length];
    configArgs[0] = -1;
    int configArgIndex = 0;
    for (int i = 0; i < args.length; i++) {
      boolean found = false;
      if (args[i].equalsIgnoreCase("-debug")) {
        this.debug = true;
      } else {
        if (args[i].equalsIgnoreCase("-nosplash")) {
          this.splashDown = true;
          found = true;
        } 
        if (args[i].equalsIgnoreCase("-noExit")) {
          System.getProperties().put("osgi.noShutdown", "true");
          found = true;
        } 
        if (args[i].equalsIgnoreCase("-initialize")) {
          this.initialize = true;
        } else if (args[i].equalsIgnoreCase("-dev") && (
          i + 1 == args.length || (i + 1 < args.length && args[i + 1]
          .startsWith("-")))) {
          this.inDevelopmentMode = true;
        } else {
          if (args[i].equalsIgnoreCase("-showsplash")) {
            this.showSplash = true;
            found = true;
            if (i + 1 < args.length && !args[i + 1].startsWith("-")) {
              configArgs[configArgIndex++] = i++;
              this.splashLocation = args[i];
            } 
          } 
          if (found) {
            configArgs[configArgIndex++] = i;
          } else if (args[i].equalsIgnoreCase("-vmargs")) {
            args[i] = null;
            i++;
            this.vmargs = new String[args.length - i];
            for (int m = 0; i < args.length; i++) {
              this.vmargs[m++] = args[i];
              args[i] = null;
            } 
          } else if (i != args.length - 1 && !args[i + 1].startsWith("-")) {
            String arg = args[++i];
            if (args[i - 1].equalsIgnoreCase("-dev")) {
              this.inDevelopmentMode = true;
              this.devClassPathProps = processDevArg(arg);
              if (this.devClassPathProps != null) {
                this.devClassPath = this.devClassPathProps
                  .getProperty("org.eclipse.osgi");
                if (this.devClassPath == null)
                  this.devClassPath = this.devClassPathProps
                    .getProperty("*"); 
              } 
            } else {
              if (args[i - 1].equalsIgnoreCase("-framework")) {
                this.framework = arg;
                found = true;
              } 
              if (args[i - 1].equalsIgnoreCase("-os")) {
                this.os = arg;
                System.getProperties().put("os.type", this.os);
              } else if (args[i - 1].equalsIgnoreCase("-ws")) {
                this.ws = arg;
              } else if (args[i - 1].equalsIgnoreCase("-arch")) {
                this.arch = arg;
              } else {
                if (args[i - 1].equalsIgnoreCase("-install")) {
                  System.getProperties().put("osgi.install.area", arg);
                  found = true;
                } 
                if (args[i - 1].equalsIgnoreCase("-configuration")) {
                  System.getProperties().put("osgi.configuration.area", arg);
                  found = true;
                } 
                if (args[i - 1].equalsIgnoreCase("-exitdata")) {
                  this.exitData = arg;
                  found = true;
                } 
                if (args[i - 1].equalsIgnoreCase("-name"))
                  found = true; 
                if (args[i - 1].equalsIgnoreCase("-startup"))
                  found = true; 
                if (args[i - 1].equalsIgnoreCase("-launcher"))
                  found = true; 
                if (args[i - 1].equalsIgnoreCase("--launcher.library")) {
                  this.library = arg;
                  found = true;
                } 
                if (args[i - 1].equalsIgnoreCase("-endsplash")) {
                  this.endSplash = arg;
                  found = true;
                } 
                if (args[i - 1].equalsIgnoreCase("-vm")) {
                  this.vm = arg;
                  found = true;
                } 
                if (args[i - 1].equalsIgnoreCase("-nl")) {
                  System.getProperties().put("osgi.nl", arg);
                  found = true;
                } 
                if (found) {
                  configArgs[configArgIndex++] = i - 1;
                  configArgs[configArgIndex++] = i;
                } 
              } 
            } 
          } 
        } 
      } 
    } 
    String[] passThruArgs = new String[args.length - configArgIndex - (
        (this.vmargs == null) ? 0 : (this.vmargs.length + 1))];
    configArgIndex = 0;
    int j = 0;
    for (int k = 0; k < args.length; k++) {
      if (k == configArgs[configArgIndex]) {
        configArgIndex++;
      } else if (args[k] != null) {
        passThruArgs[j++] = args[k];
      } 
    } 
    return passThruArgs;
  }
  
  private Properties processDevArg(String arg) {
    if (arg == null)
      return null; 
    try {
      URL location = new URL(arg);
      return load(location, null);
    } catch (MalformedURLException malformedURLException) {
      Properties result = new Properties();
      result.put("*", arg);
      return result;
    } catch (IOException iOException) {
      return null;
    } 
  }
  
  private URL getConfigurationLocation() {
    if (this.configurationLocation != null)
      return this.configurationLocation; 
    this.configurationLocation = buildLocation("osgi.configuration.area", null, 
        "");
    if (this.configurationLocation == null) {
      this.configurationLocation = buildLocation(
          "osgi.configuration.area.default", null, "");
      if (this.configurationLocation == null)
        this.configurationLocation = buildURL(
            computeDefaultConfigurationLocation(), true); 
    } 
    if (this.configurationLocation != null)
      System.getProperties().put("osgi.configuration.area", 
          this.configurationLocation.toExternalForm()); 
    if (this.debug)
      System.out
        .println("Configuration location:\n    " + this.configurationLocation); 
    return this.configurationLocation;
  }
  
  private void processConfiguration() {
    URL baseConfigurationLocation = null;
    Properties baseConfiguration = null;
    if (System.getProperty("osgi.configuration.area") == null) {
      String baseLocation = System.getProperty("osgi.baseConfiguration.area");
      if (baseLocation != null)
        baseConfigurationLocation = buildURL(baseLocation, true); 
      if (baseConfigurationLocation == null)
        try {
          baseConfigurationLocation = new URL(
              getInstallLocation(), "configuration/");
        } catch (MalformedURLException malformedURLException) {} 
      baseConfiguration = 
        loadConfiguration(baseConfigurationLocation);
      if (baseConfiguration != null) {
        String location = baseConfiguration
          .getProperty("osgi.configuration.area");
        if (location != null)
          System.getProperties().put("osgi.configuration.area", location); 
        location = baseConfiguration.getProperty("osgi.install.area");
        if (location != null && 
          System.getProperty("osgi.install.area") == null)
          System.getProperties().put("osgi.install.area", location); 
      } 
    } 
    Properties configuration = baseConfiguration;
    if (configuration == null || 
      !getConfigurationLocation().equals(
        baseConfigurationLocation))
      configuration = loadConfiguration(
          getConfigurationLocation()); 
    mergeProperties(System.getProperties(), configuration);
    if ("false".equalsIgnoreCase(System.getProperty("osgi.configuration.cascaded"))) {
      System.getProperties().remove("osgi.sharedConfiguration.area");
    } else {
      ensureAbsolute("osgi.sharedConfiguration.area");
      URL sharedConfigURL = buildLocation("osgi.sharedConfiguration.area", 
          null, "");
      if (sharedConfigURL == null)
        try {
          sharedConfigURL = new URL(getInstallLocation(), 
              "configuration/");
        } catch (MalformedURLException malformedURLException) {} 
      if (sharedConfigURL != null)
        if (sharedConfigURL.equals(getConfigurationLocation())) {
          System.getProperties().remove("osgi.sharedConfiguration.area");
        } else {
          configuration = baseConfiguration;
          if (!sharedConfigURL.equals(baseConfigurationLocation))
            configuration = loadConfiguration(sharedConfigURL); 
          mergeProperties(System.getProperties(), configuration);
          System.getProperties().put("osgi.sharedConfiguration.area", 
              sharedConfigURL.toExternalForm());
          if (this.debug)
            System.out
              .println("Shared configuration location:\n    " + sharedConfigURL.toExternalForm()); 
        }  
    } 
    String urlString = System.getProperty("osgi.framework", null);
    if (urlString != null) {
      URL url = buildURL(urlString, true);
      System.getProperties().put("osgi.framework", url.toExternalForm());
      this.bootLocation = resolve(urlString);
    } 
  }
  
  private void ensureAbsolute(String locationProperty) {
    String propertyValue = System.getProperty(locationProperty);
    if (propertyValue == null)
      return; 
    URL locationURL = null;
    try {
      locationURL = new URL(propertyValue);
    } catch (MalformedURLException malformedURLException) {
      return;
    } 
    String locationPath = locationURL.getPath();
    if (locationPath.startsWith("/"))
      return; 
    URL installURL = getInstallLocation();
    if (!locationURL.getProtocol().equals(installURL.getProtocol()))
      return; 
    try {
      URL absoluteURL = new URL(installURL, locationPath);
      System.getProperties().put(locationProperty, 
          absoluteURL.toExternalForm());
    } catch (MalformedURLException malformedURLException) {}
  }
  
  private URL getInstallLocation() {
    if (this.installLocation != null)
      return this.installLocation; 
    String installArea = System.getProperty("osgi.install.area");
    if (installArea != null) {
      this.installLocation = buildURL(installArea, true);
      if (this.installLocation == null)
        throw new IllegalStateException(
            "Install location is invalid: " + installArea); 
      System.getProperties().put("osgi.install.area", 
          this.installLocation.toExternalForm());
      if (this.debug)
        System.out
          .println("Install location:\n    " + this.installLocation); 
      return this.installLocation;
    } 
    ProtectionDomain domain = org.eclipse.equinox.launcher.Main.class.getProtectionDomain();
    CodeSource source = null;
    URL result = null;
    if (domain != null)
      source = domain.getCodeSource(); 
    if (source == null || domain == null) {
      if (this.debug)
        System.out
          .println("CodeSource location is null. Defaulting the install location to file:startup.jar"); 
      try {
        result = new URL("file:startup.jar");
      } catch (MalformedURLException malformedURLException) {}
    } 
    if (source != null)
      result = source.getLocation(); 
    String path = decode(result.getFile());
    File file = new File(path);
    path = file.toString().replace('\\', '/');
    if (File.separatorChar == '\\' && 
      Character.isUpperCase(path.charAt(0))) {
      char[] chars = path.toCharArray();
      chars[0] = Character.toLowerCase(chars[0]);
      path = new String(chars);
    } 
    System.setProperty("com.jiuqi.dna.launcher.path", file.getAbsolutePath());
    if (path.toLowerCase().endsWith(".jar"))
      path = path.substring(0, path.lastIndexOf("/") + 1); 
    if (path.toLowerCase().endsWith("/bundles/")) {
      path = path.substring(0, path.length() - "/bundles/".length());
    } else if (path.toLowerCase().endsWith("/thr/")) {
      path = path.substring(0, path.length() - "/thr/".length());
    } 
    if (path.contains("#") || path.contains("%") || path.contains("!")) {
      System.err.println("Install path: " + path);
      System.err.println("ERROR: Install path contains illegal characters like \"#%!\".");
      System.exit(1);
    } 
    try {
      try {
        path = (new File(path)).toURL().getFile();
      } catch (MalformedURLException malformedURLException) {}
      this.installLocation = new URL(result.getProtocol(), result
          .getHost(), result.getPort(), path);
      System.getProperties().put("osgi.install.area", 
          this.installLocation.toExternalForm());
    } catch (MalformedURLException malformedURLException) {}
    if (this.debug)
      System.out
        .println("Install location:\n    " + this.installLocation); 
    return this.installLocation;
  }
  
  private Properties loadConfiguration(URL url) {
    Properties result = null;
    try {
      url = new URL(url, "config.ini");
    } catch (MalformedURLException malformedURLException) {
      return null;
    } 
    try {
      if (this.debug)
        System.out.print("Configuration file:\n    " + url.toString()); 
      result = loadProperties(url);
      if (this.debug)
        System.out.println(" loaded"); 
    } catch (IOException iOException) {
      if (this.debug)
        System.out.println(" not found or not read"); 
    } 
    return result;
  }
  
  private Properties loadProperties(URL url) throws IOException {
    if (url == null)
      return null; 
    Properties result = null;
    IOException originalException = null;
    try {
      result = load(url, null);
    } catch (IOException e1) {
      originalException = e1;
      try {
        result = load(url, ".tmp");
      } catch (IOException iOException) {
        try {
          result = load(url, ".bak");
        } catch (IOException iOException1) {
          throw originalException;
        } 
      } 
    } 
    return result;
  }
  
  private Properties load(URL url, String suffix) throws IOException {
    if (suffix != null && !suffix.equals(""))
      url = new URL(url.getProtocol(), url.getHost(), url.getPort(), 
          String.valueOf(url.getFile()) + 
          suffix); 
    Properties props = new Properties();
    InputStream is = null;
    try {
      is = url.openStream();
      props.load(is);
    } finally {
      if (is != null)
        try {
          is.close();
        } catch (IOException iOException) {} 
    } 
    return props;
  }
  
  private void handleSplash(URL[] defaultPath) {
    if (this.initialize || this.splashDown || this.bridge == null) {
      this.showSplash = false;
      this.endSplash = null;
      return;
    } 
    if (this.showSplash || this.endSplash != null)
      try {
        Runtime.getRuntime().addShutdownHook(this.splashHandler);
      } catch (Throwable throwable) {} 
    if (this.endSplash != null) {
      this.showSplash = false;
      return;
    } 
    if (!this.showSplash)
      return; 
    this.splashLocation = getSplashLocation(defaultPath);
    if (this.debug)
      System.out.println("Splash location:\n    " + this.splashLocation); 
    if (this.splashLocation == null)
      return; 
    this.bridge.showSplash(this.splashLocation);
    long handle = this.bridge.getSplashHandle();
    if (handle != 0L && handle != -1L) {
      System.getProperties().put("org.eclipse.equinox.launcher.splash.handle", String.valueOf(handle));
      System.getProperties().put("org.eclipse.equinox.launcher.splash.location", this.splashLocation);
      this.bridge.updateSplash();
    } 
  }
  
  protected void takeDownSplash() {
    if (this.splashDown || this.bridge == null)
      return; 
    this.splashDown = this.bridge.takeDownSplash();
    System.getProperties().remove("org.eclipse.equinox.launcher.splash.handle");
    try {
      Runtime.getRuntime().removeShutdownHook(this.splashHandler);
    } catch (Throwable throwable) {}
  }
  
  private String getSplashLocation(URL[] bootPath) {
    if (this.splashLocation != null && 
      !Character.isDigit(this.splashLocation.charAt(0)) && (
      new File(this.splashLocation)).exists()) {
      System.getProperties()
        .put("osgi.splashLocation", this.splashLocation);
      return this.splashLocation;
    } 
    String result = System.getProperty("osgi.splashLocation");
    if (result != null)
      return result; 
    String splashPath = System.getProperty("osgi.splashPath");
    if (splashPath != null) {
      String[] entries = getArrayFromList(splashPath);
      ArrayList<String> path = new ArrayList(entries.length);
      for (int i = 0; i < entries.length; i++) {
        String entry = resolve(entries[i]);
        if (entry == null || entry.startsWith("file:")) {
          File entryFile = new File(entry.substring(5).replace('/', 
                File.separatorChar));
          entry = searchFor(entryFile.getName(), entryFile
              .getParent());
          if (entry != null)
            path.add(entry); 
        } else {
          log("Invalid splash path entry: " + entries[i]);
        } 
      } 
      result = searchForSplash(path
          .<String>toArray(new String[path.size()]));
      if (result != null) {
        System.getProperties().put("osgi.splashLocation", result);
        return result;
      } 
    } 
    result = searchSplash(bootPath, "bundles");
    if (result == null)
      result = searchSplash(bootPath, "thr"); 
    return result;
  }
  
  private String searchSplash(URL[] bootPath, String dir) {
    String result = null;
    String temp = bootPath[0].getFile();
    temp = temp.replace('/', File.separatorChar);
    int ix = temp.lastIndexOf(String.valueOf(dir) + File.separator);
    if (ix != -1) {
      int pix = temp.indexOf(File.separator, ix + 8);
      if (pix != -1) {
        temp = temp.substring(0, pix);
        result = searchForSplash(new String[] { temp });
        if (result != null)
          System.getProperties().put("osgi.splashLocation", result); 
      } 
    } 
    return result;
  }
  
  private String searchForSplash(String[] searchPath) {
    if (searchPath == null)
      return null; 
    String locale = (String)System.getProperties().get("osgi.nl");
    if (locale == null)
      locale = Locale.getDefault().toString(); 
    String[] nlVariants = buildNLVariants(locale);
    for (int i = 0; i < nlVariants.length; i++) {
      for (int j = 0; j < searchPath.length; j++) {
        if (isJAR(searchPath[j])) {
          String result = extractSplashFromJAR(searchPath[j], 
              nlVariants[i]);
          if (result != null)
            return result; 
        } else {
          String path = searchPath[j];
          if (!path.endsWith(File.separator))
            path = String.valueOf(path) + File.separator; 
          path = String.valueOf(path) + nlVariants[i];
          File result = new File(path);
          if (result.exists())
            return result.getAbsolutePath(); 
        } 
      } 
    } 
    return null;
  }
  
  private static void transferStreams(InputStream source, OutputStream destination) {
    byte[] buffer = new byte[8096];
    try {
      while (true) {
        int bytesRead = -1;
        try {
          bytesRead = source.read(buffer);
        } catch (IOException iOException) {
          try {
            source.close();
          } catch (IOException iOException1) {
            try {
              destination.close();
            } catch (IOException iOException2) {}
          } finally {
            try {
              destination.close();
            } catch (IOException iOException1) {}
          } 
        } 
        if (bytesRead == -1)
          break; 
        try {
          destination.write(buffer, 0, bytesRead);
        } catch (IOException iOException) {
          try {
            source.close();
          } catch (IOException iOException1) {
            try {
              destination.close();
            } catch (IOException iOException2) {}
          } finally {
            try {
              destination.close();
            } catch (IOException iOException1) {}
          } 
        } 
      } 
    } finally {
      try {
        source.close();
      } catch (IOException iOException) {
        try {
          destination.close();
        } catch (IOException iOException1) {}
      } finally {
        try {
          destination.close();
        } catch (IOException iOException) {}
      } 
    } 
    try {
      source.close();
    } catch (IOException iOException) {
      try {
        destination.close();
      } catch (IOException iOException1) {}
    } finally {
      try {
        destination.close();
      } catch (IOException iOException) {}
    } 
  }
  
  private String extractSplashFromJAR(String jarPath, String splashPath) {
    ZipFile file;
    OutputStream output;
    String configLocation = System.getProperty("osgi.configuration.area");
    if (configLocation == null) {
      log("Configuration area not set yet. Unable to extract splash from JAR'd plug-in: " + jarPath);
      return null;
    } 
    URL configURL = buildURL(configLocation, false);
    if (configURL == null)
      return null; 
    File splash = new File(configURL.getPath(), "com.jiuqi.dna.launcher");
    File jarFile = new File(jarPath);
    String cache = jarFile.getName();
    if (cache.endsWith(".jar"))
      cache = cache.substring(0, cache.length() - 4); 
    splash = new File(splash, cache);
    splash = new File(splash, splashPath);
    if (splash.exists()) {
      boolean clean = false;
      for (int i = 0; i < this.commands.length; i++) {
        if ("-clean".equalsIgnoreCase(this.commands[i])) {
          clean = true;
          splash.delete();
          break;
        } 
      } 
      if (!clean)
        return splash.getAbsolutePath(); 
    } 
    try {
      file = new ZipFile(jarPath);
    } catch (IOException e) {
      log("Exception looking for splash  in JAR file: " + jarPath);
      log(e);
      return null;
    } 
    ZipEntry entry = file.getEntry(splashPath.replace(File.separatorChar, 
          '/'));
    if (entry == null)
      return null; 
    InputStream input = null;
    try {
      input = file.getInputStream(entry);
    } catch (IOException e) {
      log("Exception opening splash: " + entry.getName() + " in JAR file: " + jarPath);
      log(e);
      return null;
    } 
    (new File(splash.getParent())).mkdirs();
    try {
      output = new BufferedOutputStream(new FileOutputStream(splash));
    } catch (FileNotFoundException fileNotFoundException) {
      try {
        input.close();
      } catch (IOException iOException) {}
      return null;
    } 
    transferStreams(input, output);
    return splash.exists() ? splash.getAbsolutePath() : null;
  }
  
  private boolean isJAR(String path) {
    if (path.endsWith(File.separator))
      return false; 
    int index = path.lastIndexOf('.');
    if (index == -1)
      return false; 
    index++;
    if (index >= path.length())
      return false; 
    return "JAR".equalsIgnoreCase(path.substring(index));
  }
  
  private static String[] buildNLVariants(String locale) {
    String nl = locale;
    ArrayList<String> result = new ArrayList(4);
    while (true) {
      result
        .add("nl" + File.separatorChar + nl.replace('_', File.separatorChar) + File.separatorChar + "splash.bmp");
      int lastSeparator = nl.lastIndexOf('_');
      if (lastSeparator == -1)
        break; 
      nl = nl.substring(0, lastSeparator);
    } 
    result.add("splash.bmp");
    return result.<String>toArray(new String[result.size()]);
  }
  
  private String resolve(String urlString) {
    if (urlString.startsWith("reference:")) {
      urlString = urlString.substring(10);
      System.getProperties().put("osgi.framework", urlString);
    } 
    if (urlString.startsWith("platform:/base/")) {
      String path = urlString.substring("platform:/base/".length());
      return getInstallLocation() + path;
    } 
    return urlString;
  }
  
  protected synchronized void log(Object obj) {
    if (obj == null)
      return; 
    try {
      openLogFile();
      try {
        if (this.newSession) {
          this.log.write("!SESSION");
          this.log.write(32);
          String timestamp = (new Date()).toString();
          this.log.write(timestamp);
          this.log.write(32);
          for (int i = "!SESSION".length() + timestamp.length(); i < 78; i++)
            this.log.write(45); 
          this.log.newLine();
          this.newSession = false;
        } 
        write(obj);
      } finally {
        if (this.logFile == null) {
          if (this.log != null)
            this.log.flush(); 
        } else {
          closeLogFile();
        } 
      } 
    } catch (Exception e) {
      System.err
        .println("An exception occurred while writing to the platform log:");
      e.printStackTrace(System.err);
      System.err.println("Logging to the console instead.");
      try {
        this.log = logForStream(System.err);
        write(obj);
        this.log.flush();
      } catch (Exception e2) {
        System.err
          .println("An exception occurred while logging to the console:");
        e2.printStackTrace(System.err);
      } 
    } finally {
      this.log = null;
    } 
  }
  
  private void write(Object obj) throws IOException {
    if (obj == null)
      return; 
    if (obj instanceof Throwable) {
      this.log.write("!STACK");
      this.log.newLine();
      ((Throwable)obj).printStackTrace(new PrintWriter(this.log));
    } else {
      this.log.write("!ENTRY");
      this.log.write(32);
      this.log.write("com.jiuqi.dna.launcher");
      this.log.write(32);
      this.log.write(String.valueOf(4));
      this.log.write(32);
      this.log.write(String.valueOf(0));
      this.log.write(32);
      this.log.write(getDate(new Date()));
      this.log.newLine();
      this.log.write("!MESSAGE");
      this.log.write(32);
      this.log.write(String.valueOf(obj));
    } 
    this.log.newLine();
  }
  
  protected String getDate(Date date) {
    Calendar c = Calendar.getInstance();
    c.setTime(date);
    StringBuffer sb = new StringBuffer();
    appendPaddedInt(c.get(1), 4, sb).append('-');
    appendPaddedInt(c.get(2) + 1, 2, sb).append('-');
    appendPaddedInt(c.get(5), 2, sb).append(' ');
    appendPaddedInt(c.get(11), 2, sb).append(':');
    appendPaddedInt(c.get(12), 2, sb).append(':');
    appendPaddedInt(c.get(13), 2, sb).append('.');
    appendPaddedInt(c.get(14), 3, sb);
    return sb.toString();
  }
  
  private StringBuffer appendPaddedInt(int value, int pad, StringBuffer buffer) {
    pad--;
    if (pad == 0)
      return buffer.append(Integer.toString(value)); 
    int padding = (int)Math.pow(10.0D, pad);
    if (value >= padding)
      return buffer.append(Integer.toString(value)); 
    while (padding > value && padding > 1) {
      buffer.append('0');
      padding /= 10;
    } 
    buffer.append(value);
    return buffer;
  }
  
  private void computeLogFileLocation() {
    String logFileProp = System.getProperty("osgi.logfile");
    if (logFileProp != null) {
      if (this.logFile == null || 
        !logFileProp.equals(this.logFile.getAbsolutePath())) {
        this.logFile = new File(logFileProp);
        (new File(this.logFile.getParent())).mkdirs();
      } 
      return;
    } 
    URL base = buildURL(System.getProperty("osgi.configuration.area"), false);
    if (base == null)
      return; 
    this.logFile = new File(base.getPath(), String.valueOf(Long.toString(
            System.currentTimeMillis())) + 
        ".log");
    (new File(this.logFile.getParent())).mkdirs();
    System.getProperties()
      .put("osgi.logfile", this.logFile.getAbsolutePath());
  }
  
  private int hexToByte(byte b) {
    switch (b) {
      case 48:
        return 0;
      case 49:
        return 1;
      case 50:
        return 2;
      case 51:
        return 3;
      case 52:
        return 4;
      case 53:
        return 5;
      case 54:
        return 6;
      case 55:
        return 7;
      case 56:
        return 8;
      case 57:
        return 9;
      case 65:
      case 97:
        return 10;
      case 66:
      case 98:
        return 11;
      case 67:
      case 99:
        return 12;
      case 68:
      case 100:
        return 13;
      case 69:
      case 101:
        return 14;
      case 70:
      case 102:
        return 15;
    } 
    throw new IllegalArgumentException("Switch error decoding URL");
  }
  
  private void openLogFile() throws IOException {
    computeLogFileLocation();
    try {
      this.log = new BufferedWriter(new OutputStreamWriter(
            new FileOutputStream(this.logFile.getAbsolutePath(), true), 
            "UTF-8"));
    } catch (IOException e) {
      this.logFile = null;
      throw e;
    } 
  }
  
  private BufferedWriter logForStream(OutputStream output) {
    try {
      return new BufferedWriter(new OutputStreamWriter(output, "UTF-8"));
    } catch (UnsupportedEncodingException unsupportedEncodingException) {
      return new BufferedWriter(new OutputStreamWriter(output));
    } 
  }
  
  private void closeLogFile() throws IOException {
    try {
      if (this.log != null) {
        this.log.flush();
        this.log.close();
      } 
    } finally {
      this.log = null;
    } 
  }
  
  private void mergeProperties(Properties destination, Properties source) {
    if (destination == null || source == null)
      return; 
    for (Enumeration<Object> e = source.keys(); e.hasMoreElements(); ) {
      String key = (String)e.nextElement();
      if (key.equals("osgi.frameworkClassPath")) {
        String destinationClasspath = destination
          .getProperty("osgi.frameworkClassPath");
        String sourceClasspath = source.getProperty("osgi.frameworkClassPath");
        if (destinationClasspath == null) {
          destinationClasspath = sourceClasspath;
        } else {
          destinationClasspath = String.valueOf(destinationClasspath) + 
            sourceClasspath;
        } 
        destination.put("osgi.frameworkClassPath", destinationClasspath);
        continue;
      } 
      String value = source.getProperty(key);
      if (destination.getProperty(key) == null)
        destination.put(key, value); 
    } 
  }
  
  private void setupVMProperties() {
    if (this.vm != null)
      System.getProperties().put("eclipse.vm", this.vm); 
    setMultiValueProperty("eclipse.vmargs", this.vmargs);
    setMultiValueProperty("eclipse.commands", this.commands);
  }
  
  private void setMultiValueProperty(String property, String[] value) {
    if (value != null) {
      StringBuffer result = new StringBuffer(300);
      for (int i = 0; i < value.length; i++) {
        if (value[i] != null) {
          result.append(value[i]);
          result.append('\n');
        } 
      } 
      System.getProperties().put(property, result.toString());
    } 
  }
}
